﻿local MAJOR = "SqmApi-1.0";
local MINOR = "$Revision: 1 $";

local Sqm = LibStub:NewLibrary(MAJOR, MINOR);
if not Sqm then return end;

--Lua style object creation using secret "setmetatable" and "__index" things
local function Sqm:new()
	o = {};
	setmetatable(o, self);
	self.__index = self;
	
	return o;
end;

function Sqm:StartSession(sessionName)
	o = self:new()

	o._sessionName = sessionName;
	o._enabled = true;
	o._sessionStartTime = time();
end;

function Sqm:GetEnabled()
	return self._enabled;
end;

function Sqm:SetEnabled(value)
	self._enabled = value;
end;

function Sqm:GetValue(id)
	return self[id];
end;

function Sqm:SetValue(id, value)
	if (self[id] == nil) then
		self[id] = {};
	end;

	local stype = type(value);
	if stype == "number" then
		--nop
	else if stype == "nil" then
		value = 0;
	else
		assert(false, "Invalid value type");
	end;

	self[id].value = value;
	self[id].sum = value;
	self[id].count = 1;
end;

function Sqm:Increment(id, increment)
	if (self[id] == nil) then
		self[id] = {};
	end;

	self[id].sum = (self[id].sum or 0) + increment;
	self[id].count = (self[id].count or 0) + 1;
	self[id].value = self[id].sum;
end;

function Sqm:AddToAverage(id, value)
	if (self[id] == nil) then
		self[id] = {};
	end;

	self[id].sum = (self[id].sum or 0) + value;
	self[id].count = (self[id].count or 0) + 1;
	self[id].value = self[id].sum / self[id].count;
end;

function Sqm:TimerStart(id)
	if (self[id] == nil) then
		self[id] = {};
	end;

	self[id].startTime = debugprofilestop(); --current time of profiling timer, in milliseconds
end;

function Sqm:TimerRecord(id)
	local stopTime = debugprofilestop(); --current time of profiling timer, in milliseconds

	if (self[id] == nil) then
		error("TimerStart not called");
		return; --if you didn't bother to call TimerStart then you get nothing!
	end;

	local duration = stopTime - self[id].startTime;

	self:SetValue(id, duration);
end;

function Sqm:TimerAccumulate(id)
	local stopTime = debugprofilestop(); --current time of profiling timer, in milliseconds

	if (self[id] == nil) then
		error("TimerStart not called");
		return; --if you didn't bother to call TimerStart then you get nothing!
	end;

	local duration = stopTime - self[id].startTime;

	self:Increment(id, duration);
end;

function Sqm:TimerAddToAverage(id)
	local stopTime = debugprofilestop(); --current time of profiling timer, in milliseconds

	if (self[id] == nil) then
		error("TimerStart not called");
		return; --if you didn't bother to call TimerStart then you get nothing!
	end;

	local duration = stopTime - self[id].startTime;

	self:AddToAverage(id, duration);
end;
	